function [ firstwaveTF ] = read1stwaveTF_SIMAupdated(frequency)

% this function is to generate the 1st order wave load transfer function
% for given wave frequency

% input: frequency  [rad/s]

% output: firstwaveTF (row: TF amplitude(1:6)+TF phase angle(1:6);column:each input frequency)
%                      TF amplitude [N/m]
%                      TF phase angle [rad]

% open the file
file1stwaveTF = fopen('read_SIF_updated with SIMA 1stwaveforceTF.dat','r+');
scanfile = textscan(file1stwaveTF, '%s', 'delimiter', '\n');
scanstr = scanfile{1,1};


[row, ~] = find(strcmp(scanstr, 'WAVE FREQUENCIES FORCE TRANSFER FUNCTION'), 1, 'first');

n_line = (row+(3:27))';

freq = zeros(length(n_line),1);

for j = 1:length(n_line)
freqread = scanstr(n_line(j),:);
freqsep = sscanf(freqread{1},'%f');
freq(j) = freqsep(2);

end

% read force transfer functions

str = 'FORCE TRANSFER FUNCTION';

for j = 1:length(scanstr)
    str_o=scanstr{j};
    k =  strfind(str_o,str);
    if length(k)>0
        Ind(j)=1;
    else Ind(j)=0;
    end
end

clear str j

n_l = find(Ind); % returns the line with string 'FORCE TRANSFER FUNCTION'

n_l(1:3)=[]; % n1 has line numbers for 6 dof force transfer function

for j = 1:length(n_l)
    line = (n_l(j)+(3:27))';
    for k = 1:length(line)
        value_read = scanstr(line(k),:);
        value_sep = sscanf(value_read{1},'%f');
        amp(k) = value_sep(3);  % [N/m]
        phase(k) = value_sep(4);  % [deg]
    end
        for i = 1:length(frequency)
            w = frequency(i);
            if w >= min(freq) && w <= max(freq)
                amp_TF(j,i) = interp1(freq,amp,w);
                phase_TF(j,i) = interp1(freq,phase,w);
            else
                amp_TF(j,i) = interp1(freq,amp,w,'linear','extrap'); % extrapolation for frequency smaller than the range(0-0.1Hz)
                phase_TF(j,i) = interp1(freq,phase,w,'linear','extrap'); % extrapolation for frequency smaller than the range(0-0.1Hz)
            end
        end
            clear amp phase
  
end

amp_TF_Nm = amp_TF.*1000; % in the .SIF file, the unit is KNm instead of Nm

phase_TF_rad = (phase_TF.*pi)./180; % convert the phase angle from deg into rad

firstwaveTF = zeros(6,1);

for j = 1:length(amp_TF_Nm)
    firstwaveTF(j) = amp_TF_Nm(j)*cos(phase_TF_rad(j))+ 1i*amp_TF_Nm(j)*sin(phase_TF_rad(j));
end

end

