function first_order_wave_load = read_1stwaveload(frequency,t)

% Object: this function is used to read the first order wave load from
%         the file read_SIF-dat

% open the file
file_1st = fopen('read_SIF.dat','r+');
scanfile = textscan(file_1st, '%s', 'delimiter', '\n');
scanstr = scanfile{1,1};

% read the frequencies

[row, col] = find(strcmp(scanstr, 'WAVE FREQUENCIES FORCE TRANSFER FUNCTION'), 1, 'first');

n_line1 = row+3;
n_line2 = n_line1+25;
periodread = scanstr(n_line1:n_line2,:);

for k = 1:length(periodread)
    periodread_s(k,:) = sscanf(periodread{k},'%f'); % seperate the strings in two elements
end

freq_vec =  periodread_s(:,2);

% read the first order wave force transfer function

line_nu = find(contains(scanstr,'FORCE TRANSFER FUNCTION')); % find the index of strings contains 'FORCE TRANSFER FUNCTION'
line_nu = line_nu(4:end);

for k = 1:length(line_nu)
    line_start = line_nu(k)+3;
    line_end = line_start+25;
    resource = scanstr(line_start:line_end,1);
    for j = 1:length(resource)
        resource_s(j,:) =  sscanf(resource{j},'%f');
    end
    amp_pha(:,:,k) = resource_s(:,3:4);
    
end

n_count = 1;

for k = 1:6
    if frequency <= max(freq_vec) && frequency >= min(freq_vec)
        amp(k) = interp1(freq_vec,amp_pha(:,1,k),frequency);
        phase(k)=interp1(freq_vec,amp_pha(:,2,k),frequency);
    else
        amp(k) = interp1(freq_vec,amp_pha(:,1,k),frequency,'linear','extrap'); % extrapolation for frequency outside the range
        phase(k)=interp1(freq_vec,amp_pha(:,2,k),frequency,'linear','extrap'); % extrapolation for frequency outside the range
        
    end
end
amp = amp';
amp = amp.*1e3; % change from KN to N
phase = phase';


for j = 1:6
    LTF(j) = amp(j)*exp((frequency *(t)+phase(j))*i);
    
end

LTF = transpose(LTF); %%%% use transpose instead of ' when the elements has imaginary part

% assume unit wave elevation

Z = exp((frequency*(t))*i);

first_order_wave_load = Z.*LTF;

end