function [ Jonswap_spectrum_density ] = read_wave_PSD8(frequency)

% this function if to read wave power spectrum PSD for given frequnency
% Jonswap spectrum based on SIMA RIFLEX-user-manual P132 Jonswap spectrum (IWASP1=3), one input line

% condition: wind speed = 8 m/s

% input: frequency [rad/s]
% output: Jonswap_spectrum_density [m^2s/rad]

g = 9.81; %[m/s^2]
Hs = 2.2; %[m]
Tp = 10.8; %[s]

r = 1;                    % factor Gama, Peakedness parameter, Tp>5*sqrt(Hs), Gama = 1

Tz = Tp/(1.407*((1-0.287*log(r))^(1/4)));

a = 1.2905*(Hs^2)/(Tz^4); % factor Alpha, Phillips constant

b = 1.25;                 % factor Beta, Form parameter, 1.25 for North sea condition

wp = 2*pi/Tp;


% Spectrumwidth parameter

if frequency <= wp
    sigma = 0.07;
else
    sigma = 0.09;
end

S = zeros(length(frequency),1);

for i = 1:length(frequency)
    w = frequency(i);
    if w == 0
        S(i) = 0;
    else
    S(i) = a*(g^2)*(w^(-5))*exp((-b)*(wp/w)^4)*(r^(exp(-((w-wp)^2/(2*sigma^2*wp^2)))));
    end
    clear w
end

Jonswap_spectrum_density = S;


end

