  function freqdep_addedmass_matrix = readfreaddedmass_freq_ma(frequency)

% Object: this function is used to read the frequency dependent added mass matrix from
%         the file Frequency_dependent_added_mass.txt which are generated by copying the
%         value from SIMA interface when importing G1.SIF file in SIMO.
%         the data is read from SIMO imported file, the unit is Kg and N
% open the file
filefreaddedmass = fopen('Frequency_dependent_added_mass_frequency.txt','r+');
scanfile = textscan(filefreaddedmass, '%s', 'delimiter', '\n');
scanstr = scanfile{1,1};
scanstr = regexprep(scanstr,'#.',''); % remove the '#' symbol in the string cell
scanstr = regexprep(scanstr,'.body1','body1'); % remove the space in front of the 'body1'

% convert period into frequency
[row, ~] = find(strcmp(scanstr, 'Frequency [rad/s]'), 1, 'first');

n_line = row+1;
periodread = scanstr(n_line,:);
freq_vec = sscanf(periodread{1},'%f');

line_nu = find(contains(scanstr,'Amplitude')); % find the index of strings contains 'Amplitude'
line_nu = line_nu +1;

for i = 1:length(line_nu)
    resource = scanstr{line_nu(i),1};
    resource_ma(:,i) =  sscanf(resource,'%f');
    
end

n_count = 1;

for i = 1:6
    for j = 1:6
        if frequency <= max(freq_vec) && frequency >= min(freq_vec)
            addedmass_matrix(i,j) = interp1(freq_vec,resource_ma(:,n_count),frequency); % interpolation for frequency with the range
        else
            addedmass_matrix(i,j) = interp1(freq_vec,resource_ma(:,n_count),frequency,'linear','extrap'); % extrapolation for frequency outside the range
        end
        n_count = n_count+1;
    end
end

freqdep_addedmass_matrix = addedmass_matrix;

end
