% readTimeDomainResults  Reads in results from a single simulation 
% 

close all
clear all
clc

%% User-defined input
for file = 1: 3
    if file == 1
        folder = 'H:\05.23_windonly8\windonly8\conditionSet_005s\conditionSet_005s_1\116-20180602082917';
    elseif file == 2
        folder = 'H:\05.23_windonly8\windonly8\conditionSet_005s\conditionSet_005s_2\116-20180602082917';
    elseif file == 3
        folder = 'H:\05.23_windonly8\windonly8\conditionSet_005s\conditionSet_005s_3\116-20180602082917';
   end
    
prefix = 'sima';
hullBody = 1; % typically 1, refers to the SIMA model: model-bodies-WINZIT or others

%% READ SIMULATION OUTPUT
%==========================================================================
%%READ SIMULATION OUTPUT
    % Read the SIMO results text file to get the channel names and number
    % of steps
    [nchan, nts, dt, chanNames] = readSIMO_resultstext([folder '\results.txt']);
    % Read the binary file
    BB = read_simoresults([folder '\results.tda'],nts);
    sizeBB = size(BB);
    if (sizeBB(1)<nts || nts<1); disp('Unable to read SIMO results'); return; end;
    % Determine which channels to read for the platform motions, wave
    % elevation
    [chanMotions, chanWave, chanVelocities] = getchannelNumbers(chanNames,hullBody);
    if (chanMotions(1)<1 || chanWave<1 || chanVelocities(1)<1); disp('Unable to read SIMO results'); return; end;
    time_SIMO = BB(:,2);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%chanVelocities is added by Guodong Liang%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % summarize data in matrix
    PlatMotions = BB(:,chanMotions);
    wave = BB(:,chanWave);
    PlatVelocities = BB(:,chanVelocities);

    % Read the wind turbine results 
    fname = [folder '\' prefix '_witurb.bin'];
    BB = read_rifbin(fname,0,26);
    sizeBB = size(BB);
    Nt = sizeBB(1); % get the number of time steps
    if Nt<2; disp('Unable to read RIFLEX wind turbine results'); return; end; 
    
    time_RIFLEX = BB(:,2);
    omega = BB(:,3)*pi/180; % convert from deg/s to rad/s
    genTq = BB(:,5); 
    genPwr = BB(:,6)*94.4/100; % generator efficiency included
    azimuth = BB(:,7); 
    HubWindX = BB(:,8);
    HubWindY = BB(:,9);
    HubWindZ = BB(:,10);
    AeroForceX = BB(:,11);
    AeroMomX = BB(:,14);
    Bl1Pitch = BB(:,17);
    Bl2Pitch = BB(:,18);
    Bl3Pitch = BB(:,19); 

    % Read the RIFLEX force output file
    fname = [folder '\' prefix '_elmfor.bin'];
    BB = read_rifbin(fname,Nt);
    sizeBB = size(BB);
    if(sizeBB(1)<Nt); disp('Unable to read RIFLEX force results'); return; end;  

    % Tower Base
    TowerBaseAxial  = BB(:,3);
    TowerBaseTors   = BB(:,4);
    TowerBaseBMY    = BB(:,5);
    TowerBaseBMZ    = BB(:,7);
    TowerBaseShearY = BB(:,9);
    TowerBaseShearZ = BB(:,11);

    % Tower Top
    TowerTopAxial  = BB(:,13);
    TowerTopTors   = BB(:,14);
    TowerTopBMY    = BB(:,16); % end 2
    TowerTopBMZ    = BB(:,18);
    TowerTopShearY = BB(:,20);
    TowerTopShearZ = BB(:,22);    
    
    % Blade 1
    bl1Axial        = BB(:,23);
    bl1Tors         = BB(:,24);
    bl1BMY          = BB(:,25);
    bl1BMZ          = BB(:,27);
    bl1ShearY       = BB(:,29);
    bl1ShearZ       = BB(:,31);
    
    % Blade 2
    bl2Axial        = BB(:,33);
    bl2Tors         = BB(:,34);
    bl2BMY          = BB(:,35);
    bl2BMZ          = BB(:,37);
    bl2ShearY       = BB(:,39);
    bl2ShearZ       = BB(:,41);
    
    
    clear('fid','fname','data','BB','sizeBB','chanNames','chanWave','dt')
%%END READ SIMULATION OUTPUT

%==========================================================================

%% DESCRIPTION OF TIME SERIES available in the workspace
% =========================================================================
% Platform motions and wave elevation:
%   Time variable:             time_SIMO (mx1)  : s
%   Platform positions:        PlatMotions (mx6): m,m,m,deg,deg,deg
%   Wave elevation at origin   wave(mx1)        : m 
%   Platform velocities:       PlatVelocities   : m/s,m/s,m/s,deg/s,deg/s,deg/s

% Wind turbine results : 
%   Time variable:             time_RIFLEX (jx1): s
%   Thrust (shaft x):          AeroForceX (jx1) : kN
%   Aero. torque (about shaft):AeroMomX (jx1)   : kNm (rotor inertia NOT INCLUDED) 
%   Wind speed (shaft x):      HubWindX (jx1)   : m/s (at hub pos)
%   Wind speed (shaft y):      HubWindY (jx1)   : m/s (at hub pos)
%   Wind speed (shaft z):      HubWindZ (jx1)   : m/s (at hub pos)
%   Rotor speed:               omega (jx1)      : rad/s
%   Generator power:           genPwr (jx1)     : kW (including efficiency)
%   Generator torque:          genTq (jx1)      : kNm (HSS)
%   Azimuth angle:             azimuth (jx1)    : deg 

% Internal force results at the tower base: 
%   Time variable:              time_RIFLEX (jx1)       : s
%   Axial force at tower base:  TowerBaseAxial (jx1)    : kN  (DOF1)
%   Bending mom. Y at tow base: TowerBaseBMY (jx1)      : kNm (DOF3)
%   Bending mom. Z at tow base: TowerBaseBMZ (jx1)      : kNm (DOF5)
%   Shear force Y at tow base:  TowerBaseShearY (jx1)   : kN  (DOF7)
%   Shear force Z at tow base:  TowerBaseShearZ (jx1)   : kN  (DOF9)
%   Torsional moment tow base:  TowerBaseTors (jx1)     : kNm (DOF2)

% Internal force results at the blade root for blade 1: 
%   Time variable:              time_RIFLEX (jx1)       : s
%   Axial force at tower base:  bl1Axial (jx1)          : kN  (DOF1)
%   Bending mom. Y at tow base: bl1BMY (jx1)            : kNm (DOF3)
%   Bending mom. Z at tow base: bl1BMZ (jx1)            : kNm (DOF5)
%   Shear force Y at tow base:  bl1ShearY (jx1)         : kN  (DOF7)
%   Shear force Z at tow base:  bl1ShearZ (jx1)         : kN  (DOF9)
%   Torsional moment tow base:  bl1Tors (jx1)           : kNm (DOF2)

% Internal force results at the blade root for blade 2: 
%   Time variable:              time_RIFLEX (jx1)       : s
%   Axial force at tower base:  bl2Axial (jx1)          : kN  (DOF1)
%   Bending mom. Y at tow base: bl2BMY (jx1)            : kNm (DOF3)
%   Bending mom. Z at tow base: bl2BMZ (jx1)            : kNm (DOF5)
%   Shear force Y at tow base:  bl2ShearY (jx1)         : kN  (DOF7)
%   Shear force Z at tow base:  bl2ShearZ (jx1)         : kN  (DOF9)
%   Torsional moment tow base:  bl2Tors (jx1)           : kNm (DOF2)

% =========================================================================
%% Post-processing
%  Statistical analysis, fatigue calculation, etc
%  Remember to cut out the first part of the time series
%  Hint: 
%           t_start = 400;
%           indS = sum(time_SIMO<=t_start);
%           mean(PlatMotions(indS:end))


%% take the data from steady state 
t_start = 300;
indS = sum(time_SIMO<=t_start);
mean_surge_mo(file)=mean(PlatMotions(indS:end,1));
mean_sway_mo(file)=mean(PlatMotions(indS:end,2));
mean_heave_mo(file)=mean(PlatMotions(indS:end,3));
mean_row_mo(file)=mean(PlatMotions(indS:end,4));
mean_pitch_mo(file)=mean(PlatMotions(indS:end,5));
mean_yaw_mo(file)=mean(PlatMotions(indS:end,6));

mean_surge_ve(file)=mean(PlatVelocities(indS:end,1));
mean_sway_ve(file)=mean(PlatVelocities(indS:end,2));
mean_heave_ve(file)=mean(PlatVelocities(indS:end,3));
mean_row_ve(file)=mean(PlatVelocities(indS:end,4));
mean_pitch_ve(file)=mean(PlatVelocities(indS:end,5));
mean_yaw_ve(file)=mean(PlatVelocities(indS:end,6));

end

%% post-processing

mean_heave_mo_8 = mean(mean_heave_mo(1:3));
mean_pitch_mo_8 = mean(mean_pitch_mo(1:3));
mean_row_mo_8 = mean(mean_row_mo(1:3));
mean_surge_mo_8 = mean(mean_surge_mo(1:3));
mean_sway_mo_8 = mean(mean_sway_mo(1:3));
mean_yaw_mo_8 = mean(mean_yaw_mo(1:3));

mean_heave_ve_8 = mean(mean_heave_ve(1:3));
mean_pitch_ve_8 = mean(mean_pitch_ve(1:3));
mean_row_ve_8 = mean(mean_row_ve(1:3));
mean_surge_ve_8 = mean(mean_surge_ve(1:3));
mean_sway_ve_8 = mean(mean_sway_ve(1:3));
mean_yaw_ve_8 = mean(mean_yaw_ve(1:3));

mean_mo_8 = [mean_surge_mo_8; mean_sway_mo_8; mean_heave_mo_8; mean_row_mo_8; mean_pitch_mo_8; mean_yaw_mo_8];
mean_ve_8 = [mean_surge_ve_8; mean_sway_ve_8; mean_heave_ve_8; mean_row_ve_8; mean_pitch_ve_8; mean_yaw_ve_8];

%% change the angles into rad/s (value in velocities are all zeros, no need to change)

Ind = [4 5 6];

for i=1:length(Ind)
    mean_mo_8(Ind(i)) = mean_mo_8(Ind(i))*pi/180;
end














