% This script is used to read the wind field file

%% read the result .wnd file in TurbSim

clc
clear
close all

addpath('H:\Read_Wind_Spectrum\wafo')
initwafo

formatSpec1 = 'H:\\Results for 8ms\\Seed%d\\';
formatSpec2 = 'Kaimal_8_%d';

N_smooth = 5000;

for j = 1:3
    % set the file path
    folder = sprintf(formatSpec1,j);
    wndfile = sprintf(formatSpec2,j);
    Filename = [folder wndfile];
    % the .wnd file and .sum file should be in the same folder

[velocity, y, z, nz, ny, dz, dy, dt, zHub, z1, SummVars] = readBLgrid(Filename);

[length_time, length_velocitycomp, length_iy, length_iz]= size(velocity);

%% find the index of the point within rotor plane

R = 126/2; % [m] rotor radius
n = 1;

for i = 1:ny
    for k = 1:nz
        dist(i,k) = sqrt(y(i)^2+(z(k)-90)^2);
        if dist(i,k) <= R
            ind1(i,k) = 1;
            ind(n,1) = i;
            ind(n,2) = k;
            n = n+1;
        else ind1(i,k) = 0;
        end
    end
end

clear i k
%% calculate the average of the turbulent wind speed components within rotor plane
for i = 1:length_time
    u_act = zeros(length(ind),1);
    for k = 1:length(ind)
        u_act(k) = velocity(i,1,ind(k,1),ind(k,2));     
    end
    mean_u_o(i) = mean(u_act);
    clear u_act
end

%% Calculate the rotor average wind speed spectrum

time_wind = (0.05:0.05:3600)';

mean_u = (mean_u_o(6001:78000))';         % mean_u is the longitudinal speed component


Spec_wind = dat2spec([time_wind mean_u],N_smooth);

Sp_w = struct2cell(Spec_wind);

S_wind = Sp_w{1};

f_wind = Sp_w{2}; % in rad/S

S_win(:,j) = S_wind;

clear S_wind mean_u

end

mean_S_wind = zeros(length(Sp_w{1}),1);

for j = 1 : length(mean_S_wind)
    
mean_S_wind(j) = mean(S_win(j,:));

end

%% plot the power spectrum

% load 'results_read_all_wind_8_3seeds_UPDATED.mat'

figure (1)
plot(f_wind,S_win(:,1))
title('Original power spectrum for the Input')
xlabel('f [rad/s]')
ylabel('Power spectrum density')
xlim([0 1.3])
grid on


for i =1:3
figure (2)
subplot(1,3,i)
semilogx(f_wind./(2*pi),S_win(:,i))
title('Log:Original power spectrum for the Input')
xlabel('f [Hz]')
ylabel('Power spectrum density')
xlim([0/(2*pi) 1.3/(2*pi)])
grid on
end

figure (3)
plot(f_wind,mean_S_wind)
title('Averaged power spectrum for the Input')
xlabel('f [rad/s]')
ylabel('Power spectrum density')
xlim([0 1.3])
grid on

figure (4)
graph2 = semilogx(f_wind./(2*pi),mean_S_wind);
title('Rotor-average wind speed spectrum (u = 8m/s)')
set(graph2,'LineWidth',1.5);
xlabel('Frequency [Hz]')
ylabel('Wind spectral density [m^2/s]')
xlim([0/(2*pi) 1.3/(2*pi)])
grid on
grid minor