% readTimeDomainResults  Reads in results from a single simulation
%

close all
clear all
clc

%% User-defined input
for file = 1: 4
    if file == 1
        folder = 'H:\05.23_waveonly8\waveonly8\conditionSet_005s\conditionSet_005s_1\114-20180602082439';
    elseif file == 2
        folder = 'H:\05.23_waveonly8\waveonly8\conditionSet_005s\conditionSet_005s_2\114-20180602082440';
    elseif file == 3
        folder = 'H:\05.23_waveonly8\waveonly8\conditionSet_005s\conditionSet_005s_3\114-20180602082440';
    elseif file == 4
        folder = 'H:\05.23_waveonly8\waveonly8\conditionSet_005s\conditionSet_005s_4\114-20180602082440';
    end
 
    prefix = 'sima';
    hullBody = 1; % typically 1, refers to the SIMA model: model-bodies-WINZIT or others
    
    %% READ SIMULATION OUTPUT
    %==========================================================================
    %%READ SIMULATION OUTPUT
    % Read the SIMO results text file to get the channel names and number
    % of steps
    [nchan, nts, dt, chanNames] = readSIMO_resultstext([folder '\results.txt']);
    % Read the binary file
    BB = read_simoresults([folder '\results.tda'],nts);
    sizeBB = size(BB);
    if (sizeBB(1)<nts || nts<1); disp('Unable to read SIMO results'); return; end;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% chanVelocities and chan1stWaveforces are added by Guodong Liang %%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Determine which channels to read for the platform motions, wave
    % elevation
    [chanMotions, chanWave, chanVelocities, chan1stWaveforces] = getchannelNumbers(chanNames,hullBody);
    if (chanMotions(1)<1 || chanWave<1 || chanVelocities(1)<1 || chan1stWaveforces(1)<1); disp('Unable to read SIMO results'); return; end;
    time_SIMO = BB(:,2);
    
    % summarize data in matrix
    PlatMotions = BB(:,chanMotions);
    wave = BB(:,chanWave);
    PlatVelocities = BB(:,chanVelocities);
    F1stWaveforces = BB(:,chan1stWaveforces);
    
    % Read the wind turbine results
    fname = [folder '\' prefix '_witurb.bin'];
    BB = read_rifbin(fname,0,26);
    sizeBB = size(BB);
    Nt = sizeBB(1); % get the number of time steps
    if Nt<2; disp('Unable to read RIFLEX wind turbine results'); return; end;
    
    time_RIFLEX = BB(:,2);
    omega = BB(:,3)*pi/180; % convert from deg/s to rad/s
    genTq = BB(:,5);
    genPwr = BB(:,6)*94.4/100; % generator efficiency included
    azimuth = BB(:,7);
    HubWindX = BB(:,8);
    HubWindY = BB(:,9);
    HubWindZ = BB(:,10);
    AeroForceX = BB(:,11);
    AeroMomX = BB(:,14);
    Bl1Pitch = BB(:,17);
    Bl2Pitch = BB(:,18);
    Bl3Pitch = BB(:,19);
    
    % Read the RIFLEX force output file
    fname = [folder '\' prefix '_elmfor.bin'];
    BB = read_rifbin(fname,Nt);
    sizeBB = size(BB);
    if(sizeBB(1)<Nt); disp('Unable to read RIFLEX force results'); return; end;
    
    % Tower Base
    TowerBaseAxial  = BB(:,3);
    TowerBaseTors   = BB(:,4);
    TowerBaseBMY    = BB(:,5);
    TowerBaseBMZ    = BB(:,7);
    TowerBaseShearY = BB(:,9);
    TowerBaseShearZ = BB(:,11);
    
    % Tower Top
    TowerTopAxial  = BB(:,13);
    TowerTopTors   = BB(:,14);
    TowerTopBMY    = BB(:,16); % end 2
    TowerTopBMZ    = BB(:,18);
    TowerTopShearY = BB(:,20);
    TowerTopShearZ = BB(:,22);
    
    % Blade 1
    bl1Axial        = BB(:,23);
    bl1Tors         = BB(:,24);
    bl1BMY          = BB(:,25);
    bl1BMZ          = BB(:,27);
    bl1ShearY       = BB(:,29);
    bl1ShearZ       = BB(:,31);
    
    % Blade 2
    bl2Axial        = BB(:,33);
    bl2Tors         = BB(:,34);
    bl2BMY          = BB(:,35);
    bl2BMZ          = BB(:,37);
    bl2ShearY       = BB(:,39);
    bl2ShearZ       = BB(:,41);
    
    
    clear('fid','fname','data','BB','sizeBB','chanNames','chanWave','dt')
    %%END READ SIMULATION OUTPUT
    
    %==========================================================================
    
    %% DESCRIPTION OF TIME SERIES available in the workspace
    % =========================================================================
    % Platform motions and wave elevation:
    %   Time variable:             time_SIMO (mx1)  : s
    %   Platform positions:        PlatMotions (mx6): m,m,m,deg,deg,deg
    %   Wave elevation at origin   wave(mx1)        : m
    %   Platform velocities:       PlatVelocities   : m/s,m/s,m/s,deg/s,deg/s,deg/s
    %   1st order wave force       F1stWaveforces   : kN,kN,kN,kNm,kNm,kNm
    
    % Wind turbine results :
    %   Time variable:             time_RIFLEX (jx1): s
    %   Thrust (shaft x):          AeroForceX (jx1) : kN
    %   Aero. torque (about shaft):AeroMomX (jx1)   : kNm (rotor inertia NOT INCLUDED)
    %   Wind speed (shaft x):      HubWindX (jx1)   : m/s (at hub pos)
    %   Wind speed (shaft y):      HubWindY (jx1)   : m/s (at hub pos)
    %   Wind speed (shaft z):      HubWindZ (jx1)   : m/s (at hub pos)
    %   Rotor speed:               omega (jx1)      : rad/s
    %   Generator power:           genPwr (jx1)     : kW (including efficiency)
    %   Generator torque:          genTq (jx1)      : kNm (HSS)
    %   Azimuth angle:             azimuth (jx1)    : deg
    
    % Internal force results at the tower base:
    %   Time variable:              time_RIFLEX (jx1)       : s
    %   Axial force at tower base:  TowerBaseAxial (jx1)    : kN  (DOF1)
    %   Bending mom. Y at tow base: TowerBaseBMY (jx1)      : kNm (DOF3)
    %   Bending mom. Z at tow base: TowerBaseBMZ (jx1)      : kNm (DOF5)
    %   Shear force Y at tow base:  TowerBaseShearY (jx1)   : kN  (DOF7)
    %   Shear force Z at tow base:  TowerBaseShearZ (jx1)   : kN  (DOF9)
    %   Torsional moment tow base:  TowerBaseTors (jx1)     : kNm (DOF2)
    
    % Internal force results at the blade root for blade 1:
    %   Time variable:              time_RIFLEX (jx1)       : s
    %   Axial force at tower base:  bl1Axial (jx1)          : kN  (DOF1)
    %   Bending mom. Y at tow base: bl1BMY (jx1)            : kNm (DOF3)
    %   Bending mom. Z at tow base: bl1BMZ (jx1)            : kNm (DOF5)
    %   Shear force Y at tow base:  bl1ShearY (jx1)         : kN  (DOF7)
    %   Shear force Z at tow base:  bl1ShearZ (jx1)         : kN  (DOF9)
    %   Torsional moment tow base:  bl1Tors (jx1)           : kNm (DOF2)
    
    % Internal force results at the blade root for blade 2:
    %   Time variable:              time_RIFLEX (jx1)       : s
    %   Axial force at tower base:  bl2Axial (jx1)          : kN  (DOF1)
    %   Bending mom. Y at tow base: bl2BMY (jx1)            : kNm (DOF3)
    %   Bending mom. Z at tow base: bl2BMZ (jx1)            : kNm (DOF5)
    %   Shear force Y at tow base:  bl2ShearY (jx1)         : kN  (DOF7)
    %   Shear force Z at tow base:  bl2ShearZ (jx1)         : kN  (DOF9)
    %   Torsional moment tow base:  bl2Tors (jx1)           : kNm (DOF2)
    
    % =========================================================================
    %% Post-processing
    %  Statistical analysis, fatigue calculation, etc
    %  Remember to cut out the first part of the time series
    %  Hint:
    %           t_start = 400;
    %           indS = sum(time_SIMO<=t_start);
    %           mean(PlatMotions(indS:end))
    
    
    %% 
    
    % results refer to SIMO time
    t_start = 300;
    indS = sum(time_SIMO<=t_start);
    
    surge_mo_tds(:,file)=PlatMotions(indS:end,1); % m
    sway_mo_tds(:,file)=PlatMotions(indS:end,2);  % m
    heave_mo_tds(:,file)=PlatMotions(indS:end,3); % m
    row_mo_tds(:,file)=PlatMotions(indS:end,4).*(pi/180);   % rad
    pitch_mo_tds(:,file)=PlatMotions(indS:end,5).*(pi/180); % rad
    yaw_mo_tds(:,file)=PlatMotions(indS:end,6).*(pi/180);   % rad
    
    wave_tds(:,file)=wave(indS:end);              % m
    
    surge_F1st_tds(:,file)=F1stWaveforces(indS:end,1).*1000; % N
    sway_F1st_tds(:,file)=F1stWaveforces(indS:end,2).*1000; % N
    heave_F1st_tds(:,file)=F1stWaveforces(indS:end,3).*1000; % N
    row_F1st_tds(:,file)=F1stWaveforces(indS:end,4).*1000; % Nm
    pitch_F1st_tds(:,file)=F1stWaveforces(indS:end,5).*1000; % Nm
    yaw_F1st_tds(:,file)=F1stWaveforces(indS:end,6).*1000; % Nm
    
    % results refer to REFLEX time
    indS1 = sum(time_RIFLEX<=t_start);
    
    hub_wind_tds(:,file)=HubWindX(indS1+1:end);     % m/s (at hub pos)
    thrust_tds(:,file)=AeroForceX(indS1+1:end)*1000;     % kN to N
    towerbaseBMY_tds(:,file)=TowerBaseBMY(indS1+1:end)*1000;     % kNm to Nm
    
end
