clc
clear
close all

% set the constants
global g rhow

g = 9.810000420e+00; %m/s^2  gravity acceleration
rhow = 0.1025E+04; %kg/m^3   sea water density

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%        FREQUENCY DOMAIN CALCULATION             %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load 'Full_WT_matrices.mat' 

Ndof_wt = length(B_wt_matrix)-12;  % dofs without foundation reference point on WT model,(6 motion, 6 velocity)

[Lamb_0] = constraints(Ndof_wt);

fclose all;

%%%%%% Aerodynamic part%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N_FF = N_wt_matrix(1:12,1:12);
N_FT = N_wt_matrix(1:12,13:end);
N_TF = N_wt_matrix(13:end,1:12);
N_TT = N_wt_matrix(13:end,13:end);

A_FF = A_wt_matrix(1:12,1:12);
A_FT = A_wt_matrix(1:12,13:end);
A_TF = A_wt_matrix(13:end,1:12);
A_TT = A_wt_matrix(13:end,13:end);

B_add = zeros(493,1);

B_wt = [B_wt_matrix B_add];

B_F = B_wt(1:12,:);
B_T = B_wt(13:end,:);

clear N_wt_matrix A_wt_matrix B_wt_matrix B_wt B_add

frequency = (0:0.01:1.6)';
fclose all;

%%%%%% Hydrodynamic part and connection%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% assemble the mass matrix
M_matrix = readmass_ma();
   
% assemble the linear damping matrix 
linB_matrix = readlindamping_ma();

% assemble the restoring matrix
C_matrix = readrestoring_ma(g);

I_ma = eye(6);
Z_ma = zeros(6);

K_total = zeros(493,493,length(frequency));
B_total = zeros(493,2,length(frequency));

for j = 1:length(frequency)
    
    omega = frequency(j);
    
    A_matrix = readfreaddedmass_freq_ma(omega);
    B_matrix = readfredamping_freq_ma(omega);
    
    N_S = [I_ma Z_ma; Z_ma M_matrix+A_matrix];
    A_S = [Z_ma I_ma; -C_matrix -(B_matrix+linB_matrix)];
    
    N_tot = [    N_TT         zeros(481,12)       N_TF   ;
             zeros(12,481)       N_S         zeros(12,12);
                N_FT         zeros(12,12)        N_FF   ];
    
    N_total = transpose(Lamb_0)*N_tot*Lamb_0;
    
    A_tot = [    A_TT         zeros(481,12)       A_TF   ;
             zeros(12,481)       A_S         zeros(12,12);
                A_FT         zeros(12,12)        A_FF   ];
     
    
    A_total = transpose(Lamb_0)*A_tot*Lamb_0;
    
    K_total(:,:,j)= inv(1i*omega.*N_total-A_total);
    
    firstwaveTF = read1stwaveTF_SIMAupdated(omega);
    
    B_S = [zeros(6,1) zeros(6,1); zeros(6,1)  firstwaveTF ];
    
    B_tot = [B_T;B_S;B_F];
    
    B_total(:,:,j) = transpose(Lamb_0)*B_tot;
    
    clear A_matrix B_matrix N_S A_S N_tot N_total A_tot A_total firstwaveTF B_S B_tot omega   %%%%%%%%%newly modified                                             %%%%%%%%%newly added
    fclose all;
end

fclose all;

%%%%%%%%%%%%% Pitch related analysis%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

TF_pitch_xu = zeros(1,2,length(frequency));
TF_heave_xu = zeros(1,2,length(frequency));
TF_surge_xu = zeros(1,2,length(frequency));

for j = 1:length(frequency)
    
    TF_pitch_xu(:,:,j) = K_total(493-7,:,j)*B_total(:,:,j);
    TF_heave_xu(:,:,j) = K_total(493-9,:,j)*B_total(:,:,j);
    TF_surge_xu(:,:,j) = K_total(493-11,:,j)*B_total(:,:,j);
end

fclose all;

S_xixi = zeros(length(frequency),1);
S_xx_wave = zeros(length(frequency),1);
S_xx = zeros(length(frequency),1);
S_1stfsurge = zeros(length(frequency),1);
S_1stfheave = zeros(length(frequency),1);
S_1stfpitch = zeros(length(frequency),1);
S_xx_surge = zeros(length(frequency),1);
S_xx_heave = zeros(length(frequency),1);


for j = 1:length(frequency)
    
   omega = frequency(j);
   
   S_xixi(j) = read_wave_PSD8(omega);
   
   S_xx_wave(j) = S_xixi(j)*(abs(TF_pitch_xu(1,2,j)))^2;
     
   S_xx(j) =  S_xx_wave(j);
   
   S_xx_surge(j) = S_xixi(j)*(abs(TF_surge_xu(1,2,j)))^2;
   
   S_xx_heave(j) = S_xixi(j)*(abs(TF_heave_xu(1,2,j)))^2;
   
   S_1stfsurge(j) = S_xixi(j)*(abs(B_total(493-5,2,j)))^2; 
   
   S_1stfheave(j) = S_xixi(j)*(abs(B_total(493-3,2,j)))^2; 
   
   S_1stfpitch(j) = S_xixi(j)*(abs(B_total(493-1,2,j)))^2; 
   
   clear omega
   
end

fclose all;

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%          TIME DOMAIN CALCULATION             %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

addpath('H:\FDTD8_waveonly_fullmodel\wafo')
initwafo

load 'read_simaresults_TDpost_waveonly_8005'

surge_mo=surge_mo_tds; % m
sway_mo=sway_mo_tds;  % m
heave_mo=heave_mo_tds; % m
row_mo=row_mo_tds;   % rad
pitch_mo=pitch_mo_tds; % rad
yaw_mo=yaw_mo_tds;   % rad

surge_f1st=surge_F1st_tds; % N
sway_f1st=sway_F1st_tds;  % N
heave_f1st=heave_F1st_tds; % N
row_f1st=row_F1st_tds;   % Nm
pitch_f1st=pitch_F1st_tds; % Nm
yaw_f1st=yaw_F1st_tds;   % Nm

wave_ele=wave_tds;              % m

[L, N_simu] = size(pitch_mo); % L: length of the time series; N_simulation: number of simulations

N_smooth = 3000;

%%%%%%% spectrum for surge motion  SIMO time  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

time_S = (0.05:0.05:3600)';

for j = 1 : N_simu
    Spec_surge(j) = dat2spec([time_S surge_mo(:,j)],N_smooth);
end

Spec_su = struct2cell(Spec_surge);
Sp_su = squeeze(Spec_su);

S_surge = zeros(length(Sp_su{1,1}),N_simu);

for i = 1: N_simu
    
    S_surge(:,i) = Sp_su{1,i};
    
end

mean_S_surge = zeros(length(Sp_su{1,1}),1);

for j = 1 : length(mean_S_surge)
    
mean_S_surge(j) = mean(S_surge(j,:));

end

%%%%%%% spectrum for heave motion  SIMO time %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for j = 1 : N_simu
    Spec_heave(j) = dat2spec([time_S heave_mo(:,j)],N_smooth);
end

Spec_h = struct2cell(Spec_heave);
Sp_h = squeeze(Spec_h);

S_heave = zeros(length(Sp_h{1,1}),N_simu);

for i = 1: N_simu
    
    S_heave(:,i) = Sp_h{1,i};
    
end

mean_S_heave = zeros(length(Sp_h{1,1}),1);

for j = 1 : length(mean_S_heave)
    
mean_S_heave(j) = mean(S_heave(j,:));

end

%%%%%%% spectrum for pitch motion  SIMO time   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

for j = 1 : N_simu
    Spec_pitch(j) = dat2spec([time_S pitch_mo(:,j)],N_smooth);
end

Spec_p = struct2cell(Spec_pitch);
Sp_p = squeeze(Spec_p);

S_pitch = zeros(length(Sp_p{1,1}),N_simu);

for i = 1: N_simu
    
    S_pitch(:,i) = Sp_p{1,i};
    
end

mean_S_pitch = zeros(length(Sp_p{1,1}),1);

for j = 1 : length(mean_S_pitch)
    
mean_S_pitch(j) = mean(S_pitch(j,:));

end


%%%%% spectrum for wave elevation  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for j = 1 : N_simu
    Spec_wave(j) = dat2spec([time_S wave_ele(:,j)],N_smooth);
end

Spec_w = struct2cell(Spec_wave);
Sp_w = squeeze(Spec_w);

S_wave = zeros(length(Sp_w{1,1}),N_simu);

for i = 1: N_simu
    
    S_wave(:,i) = Sp_w{1,i};
    
end

mean_S_wave = zeros(length(Sp_w{1,1}),1);

for j = 1 : length(mean_S_wave)
    
mean_S_wave(j) = mean(S_wave(j,:));

end

%%%%%%% spectrum for 1st order wave force  SIMO time  %%%%%%%%%%%%%%%%%%%%%
 
for j = 1 : N_simu
    Spec_surgf1(j) = dat2spec([time_S surge_f1st(:,j)],N_smooth);
    Spec_heavf1(j) = dat2spec([time_S heave_f1st(:,j)],N_smooth);
    Spec_pitcf1(j) = dat2spec([time_S pitch_f1st(:,j)],N_smooth);
end

Spec_suf1 = struct2cell(Spec_surgf1);
Sp_suf1 = squeeze(Spec_suf1);

Spec_hef1 = struct2cell(Spec_heavf1);
Sp_hef1 = squeeze(Spec_hef1);

Spec_pif1 = struct2cell(Spec_pitcf1);
Sp_pif1 = squeeze(Spec_pif1);

S_suf1 = zeros(length(Sp_suf1{1,1}),N_simu);
S_hef1 = zeros(length(Sp_hef1{1,1}),N_simu);
S_pif1 = zeros(length(Sp_pif1{1,1}),N_simu);

for i = 1: N_simu
    
    S_suf1(:,i) = Sp_suf1{1,i};
    S_hef1(:,i) = Sp_hef1{1,i};
    S_pif1(:,i) = Sp_pif1{1,i};
    
end

mean_S_suf1 = zeros(length(Sp_suf1{1,1}),1);
mean_S_hef1 = zeros(length(Sp_hef1{1,1}),1);
mean_S_pif1 = zeros(length(Sp_pif1{1,1}),1);

for j = 1 : length(mean_S_suf1)
    
mean_S_suf1(j) = mean(S_suf1(j,:));
mean_S_hef1(j) = mean(S_hef1(j,:));
mean_S_pif1(j) = mean(S_pif1(j,:));

end

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%         PLOT SECTION           %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%% plot pitch motion response in rad/s scale

f_p = Sp_p{2,1}; % in rad/S

figure (2)
graph2 = plot(frequency,S_xx,f_p,mean_S_pitch);
title('Pitch motion response spectra-Wave only')
set(graph2,'LineWidth',1.5);
xlabel('Frequency [rad/s]')
ylabel('Spectral density [rad^2s]')
legend('Linear dynamic model','Time-domain simulation')
xlim([0 1.3])
xticks(0:0.1:1.3)
grid on
grid minor

% a = trapz(frequency,S_xx)
% b = trapz(f_p,mean_S_pitch)

%%%% plot pitch motion response in logHz scale
f_he = Sp_h{2,1}; % in rad/S
f_su = Sp_su{2,1}; % in rad/S

figure (9)
subplot(1,2,1)
graph8 = plot(frequency,S_xx_surge,f_su,mean_S_surge);
title('Surge motion response spectra-Wave only')
set(graph8,'LineWidth',1.5);
xlabel('Frequency [rad/s]')
ylabel('Spectral density [m^2s]')
legend('Linear dynamic model','Time-domain simulation')
xlim([0 1.3])
xticks(0:0.1:1.3)
grid on
grid minor

subplot(1,2,2)
graph9 = plot(frequency,S_xx_heave,f_he,mean_S_heave);
title('Heave motion response spectra-Wave only')
set(graph9,'LineWidth',1.5);
xlabel('Frequency [rad/s]')
ylabel('Spectral density [m^2s]')
legend('Linear dynamic model','Time-domain simulation')
xlim([0 1.3])
xticks(0:0.1:1.3)
grid on
grid minor

%%%%% plot wave spectra and wind speed spectra

f_w = Sp_w{2,1}; % in rad/S

figure(4)
graph4 = plot(frequency,S_xixi,f_w,mean_S_wave);
title('Wave Spectra-Wave only')
set(graph4,'LineWidth',1.5);
ylabel('Wave spectral density  [m^2s]')
xlabel('Frequency [rad/s]')
legend ('Linear dynamic model','Time-domain simulation')
xlim([0 1.6])
xticks(0:0.1:1.6)
grid on
grid minor

%%%%% plot the 1st order wave force spectra

f_suf1 = Sp_suf1{2,1}; % in rad/S
f_hef1 = Sp_hef1{2,1}; % in rad/S
f_pif1 = Sp_pif1{2,1}; % in rad/S

figure (6)
subplot(3,1,1)
graph5 =plot(frequency,S_1stfsurge,f_suf1,mean_S_suf1);
title('Surge wave force spectra-Wave only')
set(graph5,'LineWidth',1.5);
xlabel('Frequency [rad/s]')
ylabel('Spectral density [N^2s]')
legend('Linear dynamic model','Time-domain simulation')
xlim([0 1.6])
xticks(0:0.1:1.6)
grid on
grid minor

subplot(3,1,2)
graph6 =plot(frequency,S_1stfheave,f_hef1,mean_S_hef1);
title('Heave wave force spectra-Wave only')
set(graph6,'LineWidth',1.5);
xlabel('Frequency [rad/s]')
ylabel('Spectral density [N^2s]')
legend('Linear dynamic model','Time-domain simulation')
xlim([0 1.6])
xticks(0:0.1:1.6)
grid on
grid minor

subplot(3,1,3)
graph7 =plot(frequency,S_1stfpitch,f_pif1,mean_S_pif1);
title('Pitch wave force spectra-Wave only')
set(graph7,'LineWidth',1.5);
xlabel('Frequency [rad/s]')
ylabel('Spectral density [(Nm)^2s]')
legend('Linear dynamic model','Time-domain simulation')
xlim([0 1.6])
xticks(0:0.1:1.6)
grid on
grid minor

%% time-domain statistics

for i = 1:4
pitchmean(i) = mean(pitch_mo_tds(:,i));
pitchstd(i)= std(pitch_mo_tds(:,i));
end

pitchmeanmean = mean(pitchmean);

pitchmeanstd = mean(pitchstd);
