function mass_matrix = readmass_ma()

% Object: this function is used to read the mass inertia matrix from sys-WFloat.dat file
% (use the data from sys-WFloat.dat file instead of readSIF.dat to deduct the influence from WT)
% considering buoyancy compensation, the COG, mass and inertia should be only for the floater without the WT


% read the file
filesys = fopen('sys-WFloat.dat','r');
scanfile = textscan(filesys, '%s', 'delimiter', '\n');

% find the line of the mass coefficients 
[row, ~] = find(strcmp(scanfile{1,1}, 'BODY MASS DATA'), 1, 'first');

% read the cog
n_line = row+5;
cogread = scanfile{1,1}(n_line,:);
cog = sscanf(cogread{1},'%f');

xG = cog(1);
yG = cog(2);
zG = cog(3);

% read mass and inertia
n_line = row+10;
masscoeffread = scanfile{1,1}(n_line,:);
masscoeff = sscanf(masscoeffread{1},'%f');

m= masscoeff(1);
Ixx = masscoeff(2);
Iyx = masscoeff(3);
Iyy = masscoeff(4);
Izx = -masscoeff(5);  % the one in sys-WFloat miss a '-' sign
Izy = masscoeff(6);
Izz = masscoeff(7);

% assemble the mass matrix

mass_matrix_Mg = [m        0        0       0      m*zG   -m*yG;
                  0        m        0     -m*zG      0     m*xG;
                  0        0        m      m*yG   -m*xG      0 ;
                  0      -m*zG     m*yG    Ixx     -Iyx    -Izx;
                  m*zG     0      -m*xG   -Iyx     Iyy     -Izy;
                 -m*yG    m*xG      0     -Izx    -Izy      Izz  ];
          
mass_matrix = mass_matrix_Mg.*1e3; % change the mass unit from Mg to Kg
   
   
end
        



 
