% This script is used to establish the constraint equations and connect the
% WT and platform together

% matrix index: Matrix_superscript_subscript

function [Lamb_0] = constraints(Ndof_wt)

% Ndof_wt: dofs without foundation reference point(6 motion, 6 velocity)
clc
close all


rmpath('C:\Program Files\MATLAB\R2017a\toolbox\maple')
addpath('C:\Program Files\MATLAB\R2017a\toolbox\symbolic')
restoredefaultpath
%%%%%%% which syms
%%%%%%% which assume

%x_p0, theta_p0 x_F0 theta_F0:mean position 
%x_p, theta_p x_F theta_F:real-time position

% read the mean value of the operating point from SIMA simulation

load 'Total_meanvalue_reuslts_windonly_8.mat'  %need to be the updated simulation results

x_p01 = mean_mo_8(1);
x_p02 = mean_mo_8(2);
x_p03 = mean_mo_8(3);
theta_g_p01 = mean_mo_8(4);
theta_g_p02 = mean_mo_8(5);
theta_g_p03 = mean_mo_8(6);
dx_p01_dt = mean_ve_8(1);
dx_p02_dt = mean_ve_8(2);
dx_p03_dt = mean_ve_8(3);
dtheta_g_p01_dt = mean_ve_8(4);
dtheta_g_p02_dt = mean_ve_8(5);
dtheta_g_p03_dt = mean_ve_8(6);

% n1 n2 n3 from node 'towerlow' in SIMA model and COG
n = [26.56 0 10]';
p = [-4.3 0 -7.937]';

P_p0_n0p0 = (n-p);
n1 = P_p0_n0p0(1);
n2 = P_p0_n0p0(2);
n3 = P_p0_n0p0(3);

P_p_np = P_p0_n0p0;


syms x_p1 x_p2 x_p3 theta_p1 theta_p2 theta_p3 x_F1 x_F2 x_F3 theta_F1 theta_F2 theta_F3
assume([x_p1 x_p2 x_p3 theta_p1 theta_p2 theta_p3 x_F1 x_F2 x_F3 theta_F1 theta_F2 theta_F3],'real')
syms dx_p1_dt dx_p2_dt dx_p3_dt dtheta_p1_dt dtheta_p2_dt dtheta_p3_dt dx_F1_dt dx_F2_dt dx_F3_dt dtheta_F1_dt dtheta_F2_dt dtheta_F3_dt
assume([dx_p1_dt dx_p2_dt dx_p3_dt dtheta_p1_dt dtheta_p2_dt dtheta_p3_dt dx_F1_dt dx_F2_dt dx_F3_dt dtheta_F1_dt dtheta_F2_dt dtheta_F3_dt],'real')
%%%%%%%%%%%%%%%%%%% at the step of linearization, variables in these two lines should be defined as equal as the value in the mean position

% establish variation equation of positions

[Rx_thetap01, Ry_thetap02, Rz_thetap03] = Rotation_matrix(theta_g_p01,theta_g_p02,theta_g_p03);

T_g_p0 = Rz_thetap03*Ry_thetap02*Rx_thetap01;     

T_p0_g = transpose(T_g_p0);

[Rx_thetap1, Ry_thetap2, Rz_thetap3] = Rotation_matrix(theta_p1,theta_p2,theta_p3);

T_p0_p = Rz_thetap3*Ry_thetap2*Rx_thetap1; 

T_p_p0 = transpose(T_p0_p);


T_g_p = T_g_p0*T_p0_p;

dif_T_g_p_thetap1 = diff(T_g_p,theta_p1);
dif_T_g_p_thetap2 = diff(T_g_p,theta_p2);
dif_T_g_p_thetap3 = diff(T_g_p,theta_p3);

E = [dif_T_g_p_thetap1*P_p_np dif_T_g_p_thetap2*P_p_np dif_T_g_p_thetap3*P_p_np];

x_p0 = [x_p01 x_p02 x_p03]';

x_F0 = x_p0+T_g_p0*P_p0_n0p0; % from position constraint equations eq 1

x_F01 = x_F0(1);
x_F02 = x_F0(2);
x_F03 = x_F0(3);

% establish variation equation of rotations

T_p_g = transpose(T_g_p);

dif_T_p0_p_thetap1 = diff(T_p0_p,theta_p1);
dif_T_p0_p_thetap2 = diff(T_p0_p,theta_p2);
dif_T_p0_p_thetap3 = diff(T_p0_p,theta_p3);

G1 = T_g_p0*dif_T_p0_p_thetap1*T_p_g;
G2 = T_g_p0*dif_T_p0_p_thetap2*T_p_g;
G3 = T_g_p0*dif_T_p0_p_thetap3*T_p_g;

T_g_F0 = T_g_p0;  % from rotation constraint equations eq 2 

theta_F0_F = [theta_F1 theta_F2 theta_F3]';

T_F0_F = TFromTheta (theta_F0_F);

T_g_F = T_g_F0*T_F0_F;

T_F0_g = transpose(T_g_F0);

I_3 = eye(3); % identity 3*3 matrix

[~,dT_F0_F] = dTdth (theta_F0_F);  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% These is an 'if' logical judgement in the original 'dTdth' function, but cannot 
%%%%%% be used for symbolic expression, thus the value is given and pre-judged
%%%%%% if theta_F0_F is not a zero vector, the function might need to be modified
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dif_T_F0_F_thetaF1 = dT_F0_F(:,1:3);
dif_T_F0_F_thetaF2 = dT_F0_F(:,4:6);
dif_T_F0_F_thetaF3 = dT_F0_F(:,7:9);

dif_T_F_F0_thetaF1 = transpose(dif_T_F0_F_thetaF1);
dif_T_F_F0_thetaF2 = transpose(dif_T_F0_F_thetaF2);
dif_T_F_F0_thetaF3 = transpose(dif_T_F0_F_thetaF3);

H1 = T_g_F *dif_T_F_F0_thetaF1*T_F0_g;  
H2 = T_g_F *dif_T_F_F0_thetaF2*T_F0_g;
H3 = T_g_F *dif_T_F_F0_thetaF3*T_F0_g;

Ind1=[1 1 2;2 3 3];

for j=1:length(G1)
    G(j,1)=G1(Ind1(1,j),Ind1(2,j));
    G(j,2)=G2(Ind1(1,j),Ind1(2,j));
    G(j,3)=G3(Ind1(1,j),Ind1(2,j));
    H(j,1)=H1(Ind1(1,j),Ind1(2,j));
    H(j,2)=H2(Ind1(1,j),Ind1(2,j));
    H(j,3)=H3(Ind1(1,j),Ind1(2,j));
end

% assemble the L_matrix

Z_3 = zeros(3,3);

L = [I_3 E -I_3 Z_3; Z_3 G Z_3 H];

% assemble the dif_L matrix

q = [x_p1 x_p2 x_p3 theta_p1 theta_p2 theta_p3 x_F1 x_F2 x_F3 theta_F1 theta_F2 theta_F3]';

dq_dt = [dx_p1_dt dx_p2_dt dx_p3_dt dtheta_p1_dt dtheta_p2_dt dtheta_p3_dt dx_F1_dt dx_F2_dt dx_F3_dt dtheta_F1_dt dtheta_F2_dt dtheta_F3_dt]';

dif_L = sym(zeros(6,12));

for k = 1:length(q)
    
    dif_L(:,k) = diff(L,q(k))*dq_dt;
    
end

K_init = [L zeros(6,12); dif_L L]; % the K matrix without expanding to the full dofs

% rearrange the K_init to be fitted in the global scale

K_in = K_init(:,[1:6 13:18 7:12 19:24]);

% expand K_init to K matrix

K_add = zeros(12,Ndof_wt);

K_mat_con = [K_add K_in];

% the full dofs should be x = [x_WT theta_WT diffx_WT difftheta_WT x_p theta_p diffx_p difftheta_p x_F theta_F diffx_F difftheta_F] 

% mean value at operating point
 
x_p1 = x_p01; 
x_p2 = x_p02;
x_p3 = x_p03;
theta_p1 = 0;
theta_p2 = 0;
theta_p3 = 0;
x_F1 = x_F01;
x_F2 = x_F02;
x_F3 = x_F03;

% these three value need to be substituted separately due to numerical error '0/0' in expm substitution
% theta_F1 = 0; 
% theta_F2 = 0;
% theta_F3 = 0;

dx_p1_dt = 0;
dx_p2_dt = 0;
dx_p3_dt = 0;
dtheta_p1_dt = 0;
dtheta_p2_dt = 0;
dtheta_p3_dt = 0;
dx_F1_dt = 0;
dx_F2_dt = 0;
dx_F3_dt = 0;
dtheta_F1_dt = 0;
dtheta_F2_dt = 0;
dtheta_F3_dt = 0;

% theta_F1 = theta_F2 = theta_F3 = 0;

K_1 = subs(K_mat_con);
K_2 = subs(K_1,theta_F1,0);
K_3 = subs(K_2,theta_F2,0);
K_4 = double(limit(K_3,theta_F3,0));

K_0 = K_4;



K_F0 = K_0(:,(length(K_0)-12+1):end);

K_tri0 = K_0(:,1:(length(K_0)-12));

Lamb_0 = [eye(12+Ndof_wt);(-1).*inv(K_F0)*K_tri0];

end

