function restoring_matrix = readrestoring_ma(g)

% Object: this function is used to read the restoring matrix from readSIF.dat file
% considering the buoyancy compensation, the influence of WT is also deducted.


%% read the file
fileSIF = fopen('read_SIF.dat','r');

scanfile = textscan(fileSIF, '%s', 'delimiter', '\n');

% find the line of linear stiffness
[row, ~] = find(strcmp(scanfile{1,1}, 'LINEAR STIFFNESS MATRIX'), 1, 'first');

% read the linear stiffness matrix
n_line1 = row+3;
n_line2 = n_line1+5;
linstiff_read = scanfile{1,1}(n_line1:n_line2,:);

for i = 1:6
    
    linstiff_Mg(i,:) = sscanf(linstiff_read{i},'%f');
    
end

linstiff = linstiff_Mg.*1e3;  % change from Mg to Kg

%% add the mooring line stiffness (horizontal) in the restoring matrix

% calculate natural frequency in surge, sway and yaw
T_surge = 107.0; % s
T_sway = 124.8; % s
T_yaw = 68.5; % s
omega_surge = 2*pi*(1/T_surge);
omega_sway = 2*pi*(1/T_sway);
omega_yaw = 2*pi*(1/T_yaw);

vector = [omega_surge 1; omega_sway 2; omega_yaw 6];

% from SIF.file, not sys-WFloat
xG = -3.310000000e-01;
yG = 0.000000000e+00;
zG = 1.488999960e+00 ;

m= 4.618999500e+03;
Ixx = 5.431039490e+06;
Iyx = -0.000000000e+00;
Iyy = 5.383627780e+06;
Izx = -1.262099840e+06;
Izy = -0.000000000e+00;
Izz = 3.163409660e+06;

% assemble the mass matrix

M_matrix_Mg    = [m        0        0       0      m*zG   -m*yG;
                  0        m        0     -m*zG      0     m*xG;
                  0        0        m      m*yG   -m*xG      0 ;
                  0      -m*zG     m*yG    Ixx     -Iyx    -Izx;
                  m*zG     0      -m*xG   -Iyx     Iyy     -Izy;
                 -m*yG    m*xG      0     -Izx    -Izy      Izz  ];

M_matrix = M_matrix_Mg.*1e3;

for j = 1:3
    n_freq = vector(j,1);
    A_matrix = readfreaddedmass_freq_ma(n_freq);
    A = A_matrix(vector(j,2),vector(j,2));
    M = M_matrix(vector(j,2),vector(j,2));
    C = (vector (j,1)^2)*(M+A);
    linstiff(vector(j,2),vector(j,2))=linstiff(vector(j,2),vector(j,2))+C;
end

restoring_matrix = linstiff;


restoring_matrix = linstiff;
restoring_matrix(5,5)=restoring_matrix(5,5)+249.718*1e3*g*43.4+349.9129*1e3*g*78.4493;
restoring_matrix(4,4)=restoring_matrix(4,4)+249.718*1e3*g*43.4+349.9129*1e3*g*78.4493;
restoring_matrix(4,6)=restoring_matrix(4,6)-249.718*1e3*g*26.56-349.9129*1e3*g*26.56;
restoring_matrix(5,6)=restoring_matrix(5,6)-249.718*1e3*g*0-349.9129*1e3*g*0;

end